@@ -407,8 +407,8 @@ class TestReapWorkers:
         mock_worker = mock.Mock()
         arbiter.WORKERS = {42: mock_worker}
 
-        # SIGTERM should be logged as warning (not error)
-        with mock.patch.object(arbiter.log, 'warning') as mock_log:
+        # SIGTERM should be logged as info (expected during graceful shutdown)
+        with mock.patch.object(arbiter.log, 'info') as mock_log:
             arbiter.reap_workers()
 
         # Should log the signal
